@@ -7,7 +7,6 @@ import os
 from pythran.typing import NDArray, List, Tuple
 
 
-@TestEnv.module
 class TestNumpyFunc0(TestEnv):
 
     def test_extended_sum0(self):
@@ -64,16 +63,31 @@ class TestNumpyFunc0(TestEnv):
                        numpy.ones(3, numpy.int16),
                        numpy_shape_as_function=[NDArray[numpy.int16,:]])
 
+    def test_numpy_shape_scalar(self):
+         self.run_test("def numpy_shape_scalar(a): import numpy ; return numpy.shape(a)",
+                       1.,
+                       numpy_shape_scalar=[float])
+
     def test_numpy_size_as_function(self):
          self.run_test("def numpy_size_as_function(a): import numpy ; return numpy.size(a)",
                        numpy.ones(3, numpy.int16),
                        numpy_size_as_function=[NDArray[numpy.int16,:]])
 
+    def test_numpy_size_scalar(self):
+         self.run_test("def numpy_size_scalar(a): import numpy ; return numpy.size(a)",
+                       1.,
+                       numpy_size_scalar=[float])
+
     def test_numpy_ndim_as_function(self):
          self.run_test("def numpy_ndim_as_function(a): import numpy ; return numpy.ndim(a)",
                        numpy.ones(3, numpy.int16),
                        numpy_ndim_as_function=[NDArray[numpy.int16,:]])
 
+    def test_numpy_ndim_scalar(self):
+         self.run_test("def numpy_ndim_scalar(a): import numpy ; return numpy.ndim(a)",
+                       1.,
+                       numpy_ndim_scalar=[float])
+
     def test_frexp0(self):
         self.run_test("def np_frexp0(a): import numpy as np ; return np.frexp(a)", 1.5, np_frexp0=[float])
 
@@ -267,6 +281,12 @@ def np_rosen_der(x):
     def test_median6(self):
         self.run_test("def np_median6(l): from numpy import median ; return l + median(l)", numpy.array([3, 1]), np_median6=[NDArray[int, :]])
 
+    def test_median7(self):
+        self.run_test("def np_median7(l): from numpy import median ; return median(l)", [3, 1], np_median7=[List[int]])
+
+    def test_median8(self):
+        self.run_test("def np_median8(l): from numpy import median ; return median(l)", [[3, 1],[4,5]], np_median8=[List[List[int]]])
+
     def test_mean0(self):
         self.run_test("def np_mean0(a): from numpy import mean ; return mean(a)", numpy.array([[1, 2], [3, 4]]), np_mean0=[NDArray[int,:,:]])
 
@@ -641,6 +661,10 @@ def np_ravel3(v):
     def test_resize0(self):
         self.run_test("def np_resize0(x): from numpy import resize; return resize(x, 12)", numpy.arange(24), np_resize0=[NDArray[int,:]])
 
+    def test_rollaxis4(self):
+        self.run_test("def np_rollaxis4(x): from numpy import rollaxis; return rollaxis(x, -1)",
+        numpy.array([], dtype=int), np_rollaxis4=[NDArray[int, :]])
+
     def test_rollaxis3(self):
         self.run_test("def np_rollaxis3(x): from numpy import rollaxis; return rollaxis(x, 0, 3)", numpy.arange(24).reshape((2,3,4)), np_rollaxis3=[NDArray[int, :, :, :]])
 
@@ -653,6 +677,30 @@ def np_ravel3(v):
     def test_rollaxis0(self):
         self.run_test("def np_rollaxis0(x): from numpy import rollaxis; return rollaxis(x, 1)", numpy.arange(24).reshape(2,3,4), np_rollaxis0=[NDArray[int, :, :, :]])
 
+    def test_roll12(self):
+        self.run_test("def np_roll12(x): from numpy import roll; return roll(x, (1, 2, -3), (2, 0, 1))",
+        numpy.arange(24).reshape(4, 3, 2), np_roll12=[NDArray[int, :, :, :]])
+
+    def test_roll11(self):
+        self.run_test("def np_roll11(x): from numpy import roll; return roll(x, (2, -3), (0, 0))",
+        numpy.arange(12).reshape(4, 3), np_roll11=[NDArray[int, :, :]])
+
+    def test_roll10(self):
+        self.run_test("def np_roll10(x): from numpy import roll; return roll(x, (-2, 3), (1, 0))",
+        numpy.arange(12).reshape(4, 3), np_roll10=[NDArray[int, :, :]])
+
+    def test_roll9(self):
+        self.run_test("def np_roll9(x): from numpy import roll; return roll(x, (20, 3), (0, 1))",
+        numpy.arange(12).reshape(4, 3), np_roll9=[NDArray[int, :, :]])
+
+    def test_roll8(self):
+        self.run_test("def np_roll8(x): from numpy import roll; return roll(x, 123)",
+        numpy.arange(12), np_roll8=[NDArray[int, :]])
+
+    def test_roll7(self):
+        self.run_test("def np_roll7(x): from numpy import roll; return roll(x, -1)",
+        numpy.array([], dtype=int), np_roll7=[NDArray[int, :]])
+
     def test_roll6(self):
         self.run_test("def np_roll6(x): from numpy import roll; return roll(x[:,:,:-1], -1, 2)", numpy.arange(24).reshape(2,3,4), np_roll6=[NDArray[int, :, :, :]])
 
