@@ -1,11 +1,14 @@
 import unittest
 from pythran.tests import TestEnv
 import numpy
+from packaging import version
 
 from pythran.typing import NDArray, List, Tuple
 
 
-@TestEnv.module
+np_version = version.parse(numpy.version.version)
+
+
 class TestNumpyFunc2(TestEnv):
 
     def test_nonzero0(self):
@@ -119,7 +122,7 @@ class TestNumpyFunc2(TestEnv):
         self.run_test("def np_cumprod3_(a):\n from numpy import cumprod\n return a.cumprod(1)", numpy.arange(10).reshape(2,5), np_cumprod3_=[NDArray[int,:,:]])
 
     def test_cumprod4_(self):
-        self.run_test("def np_cumprod4_(a):\n from numpy import cumprod\n return a.cumprod(0)", numpy.arange(10).reshape(2,5), np_cumprod4_=[NDArray[int,:,:]])
+        self.run_test("def np_cumprod4_(a):\n from numpy import cumprod\n return a.cumprod(0)", numpy.arange(12).reshape(2,6), np_cumprod4_=[NDArray[int,:,:]])
 
     def test_cumprod5_(self):
         self.run_test("def np_cumprod5_(a):\n from numpy import cumprod\n return a.cumprod(0)", numpy.arange(10), np_cumprod5_=[NDArray[int,:]])
@@ -174,7 +177,7 @@ class TestNumpyFunc2(TestEnv):
                   np_correlate_8=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])
 
     def test_correlate_9(self):
-        dtype = numpy.float
+        dtype = float
         self.run_test("def np_correlate_9(a,b):\n from numpy import correlate\n return correlate(a,b,'full')",
                   numpy.arange(7).astype(dtype)  + 1j*numpy.arange(7).astype(dtype),
                   numpy.arange(12).astype(dtype) + 1j*numpy.arange(12).astype(dtype),
@@ -242,7 +245,7 @@ class TestNumpyFunc2(TestEnv):
                   np_convolve_8=[NDArray[numpy.complex64,:],NDArray[numpy.complex64,:]])
 
     def test_convolve_9(self):
-        dtype = numpy.float
+        dtype = float
         self.run_test("def np_convolve_9(a,b):\n from numpy import convolve\n return convolve(a,b,'full')",
                   numpy.arange(7).astype(dtype) + 1j*  numpy.arange(7).astype(dtype),
                   numpy.arange(12).astype(dtype) + 1j* numpy.arange(12).astype(dtype),
@@ -300,6 +303,12 @@ def test_copy0(x):
     def test_clip1(self):
         self.run_test("def np_clip1(a): from numpy import  clip ; return clip(a,3,6)", numpy.arange(10), np_clip1=[NDArray[int,:]])
 
+    def test_clip2(self):
+        self.run_test("def np_clip2(a): from numpy import  clip ; return clip(a,3,None)", numpy.arange(10), np_clip2=[NDArray[int,:]])
+
+    def test_clip3(self):
+        self.run_test("def np_clip3(a): from numpy import  clip ; return clip(a,None, 6)", numpy.arange(10), np_clip3=[NDArray[int,:]])
+
     def test_concatenate0(self):
         self.run_test("def np_concatenate0(a): from numpy import array, concatenate ; b = array([[5, 6]]) ; return concatenate((a,b))", numpy.array([[1, 2], [3, 4]]), np_concatenate0=[NDArray[int,:,:]])
 
@@ -463,9 +472,11 @@ def test_copy0(x):
     def test_atleast_3d3(self):
         self.run_test("def np_atleast_3d3(a): from numpy import atleast_3d ; r = atleast_3d(a) ; return r", numpy.arange(8), np_atleast_3d3=[NDArray[int,:]])
 
+    @unittest.skipIf(np_version >= version.Version("1.16"), reason="np.asscalar is deprecated")
     def test_asscalar0(self):
         self.run_test("def np_asscalar0(a): from numpy import asscalar; return asscalar(a)", numpy.array([1], numpy.int32), np_asscalar0=[NDArray[numpy.int32,:]])
 
+    @unittest.skipIf(np_version >= version.Version("1.16"), reason="np.asscalar is deprecated")
     def test_asscalar1(self):
         self.run_test("def np_asscalar1(a): from numpy import asscalar; return asscalar(a)", numpy.array([[1]], numpy.int64), np_asscalar1=[NDArray[numpy.int64,:,:]])
 
@@ -526,6 +537,21 @@ def test_copy0(x):
     def test_array_split3(self):
         self.run_test("def np_array_split3(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a, [0,1,5])))", numpy.arange(12).reshape(6,2), np_array_split3=[NDArray[int,:,:]])
 
+    def test_array_split4(self):
+        self.run_test("def np_array_split4(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a + 1, 3)))", numpy.arange(12).reshape(6,2), np_array_split4=[NDArray[int,:,:]])
+
+    def test_array_split5(self):
+        self.run_test("""
+            def np_array_split5(a):
+                from numpy import array_split, array2string
+                split = array_split(a, 3)
+                split[0][:] = 1
+                return list(map(array2string, split)), a""",
+            numpy.arange(12).reshape(6,2), np_array_split5=[NDArray[int,:,:]])
+
+    def test_array_split6(self):
+        self.run_test("def np_array_split6(a): from numpy import array_split, array2string ; return list(map(array2string,array_split(a[1:-1], 3)))", numpy.arange(22).reshape(11,2), np_array_split6=[NDArray[int,:,:]])
+
     def test_array_equiv0(self):
         self.run_test("def np_array_equiv0(a): from numpy import array_equiv ;  b = [1,2] ; return array_equiv(a,b)", [1, 2], np_array_equiv0=[List[int]])
 
@@ -593,6 +619,15 @@ def test_copy0(x):
     def test_argsort1(self):
         self.run_test("def np_argsort1(x): return x.argsort()", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort1=[NDArray[int,:,:]])
 
+    def test_argsort2(self):
+        self.run_test("def np_argsort2(x): return x.argsort(axis=0)", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort2=[NDArray[int,:,:]])
+
+    def test_argsort3(self):
+        self.run_test("def np_argsort3(x): return x.argsort(axis=1)", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort3=[NDArray[int,:,:]])
+
+    def test_argsort4(self):
+        self.run_test("def np_argsort4(x): return x.argsort(axis=None)", numpy.array([[3, 1, 2], [1 , 2, 3]]), np_argsort4=[NDArray[int,:,:]])
+
     def test_argmax0(self):
         self.run_test("def np_argmax0(a): return a.argmax()", numpy.arange(6).reshape(2,3), np_argmax0=[NDArray[int,:,:]])
 
@@ -692,6 +727,9 @@ def test_copy0(x):
     def test_array2D_(self):
         self.run_test("def np_array2D_(a):\n from numpy import array\n return array(a)", [[1,2],[3,4]], np_array2D_=[List[List[int]]])
 
+    def test_array_iter(self):
+        self.run_test("def np_array_iter():\n from numpy import array\n return array(list(reversed(range(0, 300)))) + 3", np_array_iter=[])
+
     def test_array_typed(self):
         self.run_test("def np_array_typed(a):\n from numpy import array, int64\n return array(a, int64)", [1.,2.,3.], np_array_typed=[List[float]])
 
@@ -746,18 +784,27 @@ def test_copy0(x):
     def test_empty_integral_shape(self):
         self.run_test("def np_empty_integral_shape(n):\n from numpy import empty, uint8\n a = empty(5)\n return a.strides, len(a), n", 3, np_empty_integral_shape=[int])
 
+    def test_empty_empty_shape(self):
+        self.run_test("def np_empty_empty_shape(n):\n from numpy import empty, uint8, shape, ndim\n a = empty(())\n return a.strides, n, shape(a), ndim(a)", 3, np_empty_empty_shape=[int])
+
     def test_ones_uint_shape(self):
         self.run_test("def np_ones_uint_shape(a):\n from numpy import ones, uint32\n a = ones((uint32(a), uint32(a)))\n return a.strides, len(a)", 3, np_ones_uint_shape=[int])
 
     def test_ones_integral_shape(self):
         self.run_test("def np_ones_integral_shape(n):\n from numpy import ones, uint8\n a = ones(5)\n return a.strides, len(a), n", 3, np_ones_integral_shape=[int])
 
+    def test_ones_empty_shape(self):
+        self.run_test("def np_ones_empty_shape(n):\n from numpy import ones, uint8, shape, ndim\n a = ones(())\n return a.strides, n, shape(a), ndim(a)", 3, np_ones_empty_shape=[int])
+
     def test_zeros_uint_shape(self):
         self.run_test("def np_zeros_uint_shape(a):\n from numpy import zeros, int32\n a = zeros((int32(a), int32(a)))\n return a.strides, len(a)", 3, np_zeros_uint_shape=[int])
 
     def test_zeros_integral_shape(self):
         self.run_test("def np_zeros_integral_shape(n):\n from numpy import zeros, uint8\n a = zeros(5)\n return a.strides, len(a), n", 3, np_zeros_integral_shape=[int])
 
+    def test_zeros_empty_shape(self):
+        self.run_test("def np_zeros_empty_shape(n):\n from numpy import zeros, uint8, shape, ndim\n a = zeros(())\n return a.strides, n, shape(a), ndim(a), a.shape, a.ndim", 3, np_zeros_empty_shape=[int])
+
     def test_empty_kwargs(self):
         self.run_test("def np_empty_kwargs(a):\n from numpy import empty\n a = empty(a, dtype=int)\n return a.strides, len(a)", (3, 2), np_empty_kwargs=[Tuple[int, int]])
 
