@@ -8,7 +8,6 @@ from pythran.typing import List, NDArray, Tuple
 from pythran.tests import TestEnv
 
 
-@TestEnv.module
 class TestNumpyFunc3(TestEnv):
 
     """
@@ -527,6 +526,11 @@ def np_trim_zeros2(x):
                       numpy.array([[1,2], [7, 8]]), numpy.array([3,4]),
                       np_copyto1=[NDArray[int, :, :], NDArray[int, :]])
 
+    def test_copyto_2(self):
+        self.run_test("def np_copyto2(x, y): from numpy import copyto; return copyto(x.T, y), x",
+                      numpy.array([[1,2], [7, 8]]), numpy.array([3,4]),
+                      np_copyto2=[NDArray[int, :, :], NDArray[int, :]])
+
     def test_numpy_pow0(self):
         self.run_test('def numpy_pow0(a): return a ** 2',
                       numpy.arange(100).reshape((10, 10)),
@@ -699,121 +703,122 @@ def np_trim_zeros2(x):
 
     def test_interp_0(self):
         self.run_test('def interp0(x,xp,fp): import numpy as np; return np.interp(x,xp,fp)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7,
                       interp0=[NDArray[float,:],NDArray[float,:],NDArray[float,:]])
     def test_interp_1(self):
         self.run_test('def interp1(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,-10.,10.)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7,
                       interp1=[NDArray[float,:],NDArray[float,:],NDArray[float,:]])
     def test_interp_2(self):
         self.run_test('def interp2(x,xp,fp): import numpy as np; return np.interp(x,xp[::2],fp[::2],-10.,10.)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7,
                       interp2=[NDArray[float,:],NDArray[float,:],NDArray[float,:]])
     def test_interp_3(self):
         self.run_test('def interp3(x,xp,fp): import numpy as np; return np.interp(x[::3],xp[::2],fp[::2],-10.)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7,
                       interp3=[NDArray[float,:],NDArray[float,:],NDArray[float,:]])
+
     def test_interp_4(self):
-        self.run_test('def interp4(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,period=1.1)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+        self.run_test('def interp4(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,period=2.1)',
+                      numpy.arange(-10.,3000),
+                      numpy.arange(2000.),
+                      numpy.cos(numpy.arange(2000.)*2*numpy.pi/2.1),
                       interp4=[NDArray[float,:],NDArray[float,:],NDArray[float,:]])
 
     def test_interp_5(self):
         self.run_test('def interp5(x,factor): N = len(x); import numpy as np; return np.interp(np.arange(0, N - 1, factor), np.arange(N), x, None, None, None)',
-                      numpy.random.randn(100),
+                      numpy.arange(100.) / .5,
                       10.,
                       interp5=[NDArray[float,:],float])
-                      
+
     def test_interp_6(self):
         self.run_test('def interp6(x,xp,fp): import numpy as np; return np.interp(x,xp,fp)',
                       1.4,
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7,
                       interp6=[float,NDArray[float,:],NDArray[float,:]])
 
     def test_interp_7(self):
         self.run_test('def interp7(x,xp,fp): import numpy as np; return np.interp(x,xp,fp)',
                       1,
-                      numpy.sort(10*numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7,
                       interp7=[int,NDArray[float,:],NDArray[float,:]])
 
     def test_interp_8(self):
         self.run_test('def interp8(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,-10,10)',
                       1.4,
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000),
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7,
                       interp8=[float,NDArray[float,:],NDArray[float,:]])
 
     def test_interp_0c(self):
         self.run_test('def interp0c(x,xp,fp): import numpy as np; return np.interp(x,xp,fp)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7 + numpy.arange(1000.) * .6j,
                       interp0c=[NDArray[float,:],NDArray[float,:],NDArray[complex,:]])
-                      
+
     def test_interp_1c(self):
         self.run_test('def interp1c(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,-10.j,10.j)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7 + numpy.arange(1000.) * .6j,
                       interp1c=[NDArray[float,:],NDArray[float,:],NDArray[complex,:]])
-                      
+
     def test_interp_2c(self):
         self.run_test('def interp2c(x,xp,fp): import numpy as np; return np.interp(x,xp[::2],fp[::2],-10.,10.)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7 + numpy.arange(1000.) * .6j,
                       interp2c=[NDArray[float,:],NDArray[float,:],NDArray[complex,:]])
     def test_interp_3c(self):
         self.run_test('def interp3c(x,xp,fp): import numpy as np; return np.interp(x[::3],xp[::2],fp[::2],-10.j)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+                      numpy.arange(100.) / .5,
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7 + numpy.arange(1000.) * .6j,
                       interp3c=[NDArray[float,:],NDArray[float,:],NDArray[complex,:]])
-                      
+
     def test_interp_4c(self):
-        self.run_test('def interp4c(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,period=1.1)',
-                      numpy.random.randn(100),
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+        self.run_test('def interp4c(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,period=1.12)',
+                      numpy.arange(-10.,30,.67),
+                      numpy.arange(20.),
+                      numpy.cos(numpy.arange(20.)*2*numpy.pi/1.12) * (1+.1j),
                       interp4c=[NDArray[float,:],NDArray[float,:],NDArray[complex,:]])
 
     def test_interp_5c(self):
         self.run_test('def interp5c(x,factor): N = len(x); import numpy as np; return np.interp(np.arange(0, N - 1, factor), np.arange(N), x, None, None, None)',
-                      numpy.random.randn(100) + 1j*numpy.random.randn(100),
+                      numpy.arange(100.) / .7 + numpy.arange(100.) * .6j,
                       10.,
                       interp5c=[NDArray[complex,:],float])
-                      
+
     def test_interp_6c(self):
         self.run_test('def interp6c(x,xp,fp): import numpy as np; return np.interp(x,xp,fp)',
                       1.4,
-                      numpy.sort(numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7 + numpy.arange(1000.) * .6j,
                       interp6c=[float,NDArray[float,:],NDArray[complex,:]])
 
     def test_interp_7c(self):
         self.run_test('def interp7c(x,xp,fp): import numpy as np; return np.interp(x,xp,fp)',
                       1,
-                      numpy.sort(10*numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7 + numpy.arange(1000.) * .6j,
                       interp7c=[int,NDArray[float,:],NDArray[complex,:]])
 
     def test_interp_8c(self):
         self.run_test('def interp8c(x,xp,fp): import numpy as np; return np.interp(x,xp,fp,-10j,10j)',
                       1.4,
-                      numpy.sort(10*numpy.random.randn(1000)),
-                      numpy.random.randn(1000) + 1j*numpy.random.randn(1000),
+                      numpy.arange(1000.) / .3,
+                      numpy.arange(1000.) / .7 + numpy.arange(1000.) * .6j,
                       interp8c=[float,NDArray[float,:],NDArray[complex,:]])
 
     def test_setdiff1d0(self):
