@@ -5,6 +5,8 @@ import pythran
 from textwrap import dedent
 
 from pythran.typing import List, Dict, NDArray, Tuple
+import pythran.types.tog
+
 
 class TestTyping(TestEnv):
 
@@ -271,6 +273,24 @@ def functional_variant_container2(i):
     return l[i](1.)'''
         self.run_test(code, 4, functional_variant_container2=[int])
 
+    def test_variant_call_with_reference(self):
+        code = '''
+def variant_call_with_reference(modes, method):
+    if method == "mcma":
+        errorfct = cma_error
+    elif method == "cma":
+        errorfct = mcma_error
+    mode = modes[0]
+    err = errorfct(mode)
+    return err
+
+def cma_error(i):
+    return i
+
+def mcma_error(i):
+    return i'''
+        self.run_test(code, np.array([1,2]), "cma", variant_call_with_reference=[NDArray[int, :], str])
+
     @unittest.skip("bad typing: need backward propagation")
     def test_type_set_in_loop(self):
         code = '''
@@ -626,6 +646,14 @@ def recursive_interprocedural_typing2(c):
               return foo({x:x})'''
         return self.run_test(code, 3, rvalue_type_update_dict=[int])
 
+    @unittest.skip("bad typing: need fine grain subscript propagation")
+    def test_rvalue_type_update_dict_dyn(self):
+        code = '''
+          def rvalue_type_update_dict_dyn(x, y):
+              def foo(x): x[y] = 1.5; return x
+              return foo({x:x})'''
+        return self.run_test(code, 3, 4.5, rvalue_type_update_dict_dyn=[int, float])
+
     def test_numpy_array_combiner0(self):
         code = '''
             import numpy as np
